Imports System.IO

Public Class SClient
    Private NetStream As System.Net.Sockets.NetworkStream   'Network Stream object
    Private TcpClient As System.Net.Sockets.TcpClient       'TcpClient object

    ''' <summary>
    ''' Sends message to EI
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSend.Click
        Dim ip As String        'IP address
        Dim pn As Integer       'Port number
        Dim cmd As String       'Command
        Dim response As String  'Response from EasyInspector

        Try
            ip = Me.txtIPAddr.Text
            pn = CInt(Me.txtPort.Text)
            cmd = Me.txtCommand.Text

            response = Communicate(ip, pn, cmd, 0, True)
            Me.lstResp.Items.Insert(0, response)

        Catch ex As Exception
            MsgBox(ex.Message)

        End Try
    End Sub

    ''' <summary>
    ''' Executes EI
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnBoot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBoot.Click
        'Executable file does not exit.
        If Not IO.File.Exists(Me.txtExePath.Text) Then
            MsgBox("Executable path not found", MsgBoxStyle.Exclamation)
            Exit Sub
        End If

        Dim ei100ret As String
        Dim psi As ProcessStartInfo

        psi = New ProcessStartInfo()
        psi.FileName = Me.txtExePath.Text
        psi.UseShellExecute = False
        psi.CreateNoWindow = True
        psi.RedirectStandardOutput = False
        psi.WorkingDirectory = Path.GetDirectoryName(Me.txtExePath.Text)


        'Boot
        Process.Start(psi)
        'Wait for EI to boot and open communication. 
        Do
            ei100ret = Communicate(Me.txtIPAddr.Text, CInt(Me.txtPort.Text), "S", 20, True)
            If ei100ret <> "" Then
                Exit Do
            Else
                If MsgBox("No response from EI.  Do you want to continue?" _
                    , MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                    'retry
                Else
                    'giveup
                    Exit Sub
                End If

            End If

            Threading.Thread.Sleep(500)
        Loop
    End Sub

    ''' <summary>
    ''' Sends a command to EI and receives its response.
    ''' </summary>
    ''' <param name="ipaddr">IP address</param>
    ''' <param name="portno">Port number</param>
    ''' <param name="sendstr">Command string</param>
    ''' <param name="retrymax">Max. number of retries</param>
    ''' <param name="showerror">True: Shows error dialog if communication failed.</param>
    ''' <returns>Response from EI</returns>
    ''' <remarks></remarks>
    Public Function Communicate(ByVal ipaddr As String, ByVal portno As Integer, ByVal sendstr As String, ByVal retrymax As Integer, ByVal showerror As Boolean) As String
        Dim rtry As Integer = 0 'Retry counter
        Dim response As String  'Response from EasyInspector

        Do
            Try
                response = TcpSendRecv(ipaddr, portno, sendstr)

                If response <> "" Then
                    Return response
                Else
                    rtry += 1
                    If rtry > retrymax Then
                        If showerror = True Then
                            MsgBox("Communication failed check the communication settings of EasyInspector." & vbCrLf _
                                & "IP address = " & ipaddr & vbCrLf _
                                & "Port No. = " & portno, MsgBoxStyle.Information)
                        End If
                        Return ""

                    End If

                    Threading.Thread.Sleep(500)
                End If

            Catch ex As Exception
                MsgBox(ex.Message)

            End Try
        Loop

    End Function

    ''' <summary>
    ''' Sends the "command" to ther server specified in "ipaddr" and "port", and returns the response from the server.
    ''' </summary>
    ''' <param name="ipaddr">Target IP address</param>
    ''' <param name="port">Target Port number</param>
    ''' <param name="command">Command to send</param>
    ''' <returns>Response from server</returns>
    ''' <remarks></remarks>
    Friend Function TcpSendRecv(ByVal ipaddr As String, ByVal port As Integer, ByVal command As String) As String
        Dim RBuff(65535) As Byte        'Receive Buffer
        Dim SBuff As Byte()             'Send Buffer
        Dim StrLen As Integer           'Lengh of string
        Dim RStr As String              'Received String

        TcpClient = New System.Net.Sockets.TcpClient
        With TcpClient
            Try
                TcpClient.Connect(ipaddr, port)
                NetStream = .GetStream
                With NetStream
                    SBuff = System.Text.Encoding.Default.GetBytes(command)
                    .Write(SBuff, 0, SBuff.Length)
                    Do
                        System.Threading.Thread.Sleep(10)

                        If .DataAvailable Then
                            StrLen = .Read(RBuff, 0, RBuff.Length)
                            RStr &= System.Text.Encoding.Default.GetString(RBuff).Substring(0, StrLen)
                        ElseIf Not RStr Is Nothing Then
                            Exit Do
                        End If
                    Loop
                End With

            Catch Ex As Exception
                RStr = ""
                'MsgBox(Ex.Message)
            End Try

            .Close()
            Return RStr

        End With
    End Function

    ''' <summary>
    ''' Executable file browse button
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowse.Click
        'Create instance of OpenFileDialog
        Dim ofd As New OpenFileDialog()
        ofd.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop)
        ofd.Filter = "Executable Files(*.exe)|*.exe"
        ofd.FilterIndex = 1
        ofd.Title = "Choose EasyInspector.exe"
        ofd.RestoreDirectory = True
        ofd.CheckFileExists = True
        ofd.CheckPathExists = True

        'Write the path
        If ofd.ShowDialog() = DialogResult.OK Then
            Me.txtExePath.Text = ofd.FileName
        End If
    End Sub
End Class
